use crate::fleck_core;

#[test_log::test]
fn end_to_end() {
    tracing_log::LogTracer::init().unwrap();
    let skeleton = super::TestSkeleton::new();

    let span_f1 = tracing::span!(tracing::Level::TRACE, "f1");
    let span_f2 = tracing::span!(tracing::Level::TRACE, "f2");

    let f1 = crate::API::new();
    let f2 = crate::API::new();

    let f1pk = f1.with_service(|ns: &fleck_core::NodeService| {
        ns.build_ephemeral_self_node();
        ns.self_node().pubkey().clone()
    });
    let f2pk = f2.with_service(|ns: &fleck_core::NodeService| {
        ns.build_ephemeral_self_node();
        ns.self_node().pubkey().clone()
    });

    f1.with_service(|io: &fleck_core::io::IOService| {
        fleck_core::io::UnixSocketBuilder::default()
            .set_path(skeleton.socket_path().to_str().unwrap())
            .serve_mode()
            .build(io);
    });

    f2.with_service(|pd: &fleck_core::discovery::PeerDiscovery| {
        pd.new_peer(
            f2.with_service(|io: &fleck_core::io::IOService| {
                fleck_core::io::UnixSocketBuilder::default()
                    .set_path(skeleton.socket_path().to_str().unwrap())
                    .connect_mode()
                    .build(io)
            })
            .connected_peer(),
        )
    });

    // run for a few rounds of the event loops, without firing any ticks.
    for _ in 0..5 {
        {
            let _guard = span_f1.enter();
            f1.run_once();
        }
        {
            let _guard = span_f2.enter();
            f2.run_once();
        }
    }

    // check that they know about each other.
    f1.with_service(|ns: &fleck_core::NodeService| {
        assert!(ns.node_by_pubkey(&f2pk).is_some());
    });
    f2.with_service(|ns: &fleck_core::NodeService| {
        assert!(ns.node_by_pubkey(&f1pk).is_some());
    });
}
